@@ -1,9 +1,9 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX environment-specific functions.
 ;;; yatexenv.el
-;;; (c) 1994-2006 by HIROSE Yuuji.[yuuji@yatex.org]
-;;; Last modified Sat Jun 24 08:14:11 2006 on firestorm
-;;; $Id: yatexenv.el,v 1.73 2006/12/24 06:17:15 yuuji Rel $
+;;; (c) 1994-2012 by HIROSE Yuuji.[yuuji@yatex.org]
+;;; Last modified Sun Feb 12 11:15:33 2012 on firestorm
+;;; $Id: yatexenv.el,v 1.76 2012/05/14 10:55:57 yuuji Rel $
 
 ;;;
 ;; Functions for tabular environment
@@ -134,12 +134,10 @@ Return the list of (No.ofCols PointEndofFormat)"
 	     ((eq type 'alignat)
 	      (max
 	       1
-	       (1-
-		(* 2
-		   (string-to-int
-		    (buffer-substring
-		     (point)
-		     (progn (up-list -1) (forward-list 1) (1- (point)))))))))
+	       (* 2 (string-to-int
+		     (buffer-substring
+		      (point)
+		      (progn (up-list -1) (forward-list 1) (1- (point))))))))
 	     (t
 	      (YaTeX-tabular-parse-format-count-cols (point) eoform))))
       (list cols (1+ eoform)))))
@@ -174,25 +172,29 @@ Return the list of (No.ofCols PointEndofFormat)"
 (fset 'YaTeX-intelligent-newline-array 'YaTeX-intelligent-newline-tabular)
 (fset 'YaTeX-intelligent-newline-supertabular 'YaTeX-intelligent-newline-tabular)
 
-(defun YaTeX-intelligent-newline-alignat ()
-  (YaTeX-intelligent-newline-tabular 'alignat))
-(fset 'YaTeX-intelligent-newline-alignat* 'YaTeX-intelligent-newline-alignat)
-
 (defun YaTeX-intelligent-newline-align ()
   "Intelligent newline function for align.
 Count the number of & in the first align line and insert that many &s."
-  (let*((p (point)) (cols 0))
+  (let*((p (point)) (amps 0))
+    (if (string-match "alignat" env)
+	(setq amps (1- (car (YaTeX-tabular-parse-format 'alignat))))
+      (save-excursion
+	(YaTeX-beginning-of-environment)
+	(catch 'done
+	  (while (YaTeX-re-search-active-forward
+		  "\\(&\\)\\|\\(\\\\\\\\\\)" YaTeX-comment-prefix p t)
+	    (if (match-beginning 1) (setq amps (1+ amps)) (throw 'done t))))))
     (save-excursion
-      (YaTeX-beginning-of-environment)
-      (catch 'done
-	(while (YaTeX-re-search-active-forward
-		"\\(&\\)\\|\\(\\\\\\\\\\)" YaTeX-comment-prefix p t)
-	  (if (match-beginning 1) (setq cols (1+ cols)) (throw 'done t)))))
+      (forward-line -1)
+      (skip-chars-forward " \t")
+      (or (prog1 (looking-at "\\\\begin{") (end-of-line))
+	  (save-excursion
+	    (skip-chars-backward " \t")
+	    (and (= (preceding-char) ?\\) (= (char-after (- (point) 2)) ?\\)))
+	  (insert "\\\\")))
     (save-excursion
-      (if (= cols 0)
-	  (insert "&")
-	(while (>= (setq cols (1- cols)) 0)
-	  (insert "& "))))
+      (while (>= (setq amps (1- amps)) 0)
+	(insert "& ")))
     (YaTeX-indent-line)))
 
 (mapcar
@@ -201,7 +203,7 @@ Count the number of & in the first align line and insert that many &s."
 			   (symbol-name s)))
 	  'YaTeX-intelligent-newline-align))
  '(align* flalign  flalign* matrix pmatrix bmatrix Bmatrix vmatrix Vmatrix
-   cases))
+   cases eqnarray eqnarray* alignat alignat*))
 
 ;;;
 ;; Functions for tabbing environment
@@ -227,8 +229,7 @@ Count the number of & in the first align line and insert that many &s."
 	      (setq tabcount (1- tabcount))))
 	  (forward-char 2))
       (insert "\\= \\\\")
-      (forward-char -5)))
-)
+      (forward-char -5))))
 
 ;;;
 ;; Functions for itemize/enumerate/list environments
@@ -237,23 +238,39 @@ Count the number of & in the first align line and insert that many &s."
 (defun YaTeX-intelligent-newline-itemize ()
   "Insert '\\item '."
   (insert "\\item ")
-  (YaTeX-indent-line)
-)
+  (YaTeX-indent-line))
+
 (fset 'YaTeX-intelligent-newline-enumerate 'YaTeX-intelligent-newline-itemize)
 (fset 'YaTeX-intelligent-newline-list 'YaTeX-intelligent-newline-itemize)
 
 (defun YaTeX-intelligent-newline-description ()
   (insert "\\item[] ")
   (forward-char -2)
-  (YaTeX-indent-line)
-)
+  (YaTeX-indent-line))
 
 (defun YaTeX-intelligent-newline-thebibliography ()
   "Insert '\\bibitem '."
   (YaTeX-indent-line)
   (YaTeX-make-section nil nil nil "bibitem")
-  (YaTeX-indent-line)
-)
+  (YaTeX-indent-line))
+
+;;;
+;; For document environment
+;;;
+(defun YaTeX-intelligent-newline-document ()
+  "New paragraph by null line or `\\par'."
+  (if (< (count-lines
+	  (or (get 'YaTeX-inner-environment 'point)
+	      (max 1 (- (point) 17)))	;"\begin{document}\n" == 17
+	  (point))
+	 2)
+      nil
+    (if (save-excursion (re-search-backward "\\\\par\\>" nil t))
+	(progn
+	  (YaTeX-indent-line)
+	  (insert "\\par")))
+    (newline))
+  (YaTeX-indent-line))
 
 ;;;
 ;; Intelligent newline
