@@ -1,9 +1,9 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX and yahtml common libraries, general functions and definitions
 ;;; yatexlib.el
-;;; (c)1994-2006 by HIROSE Yuuji.[yuuji@yatex.org]
-;;; Last modified Sun Dec 24 15:12:20 2006 on firestorm
-;;; $Id: yatexlib.el,v 1.73 2006/12/24 06:17:15 yuuji Rel $
+;;; (c)1994-2012 by HIROSE Yuuji.[yuuji@yatex.org]
+;;; Last modified Wed Feb 29 09:49:30 2012 on firestorm
+;;; $Id: yatexlib.el,v 1.76 2012/05/14 10:55:57 yuuji Rel $
 
 ;; General variables
 (defvar YaTeX-dos (memq system-type '(ms-dos windows-nt OS/2)))
@@ -313,8 +313,9 @@ history list variable."
 
 ;;;###autoload
 (defun YaTeX-define-begend-key (key env &optional map)
-  "Define short cut key for begin type completion both for normal
-and region mode.  To customize YaTeX, user should use this function."
+  "Define short cut key for begin type completion.
+Define both strokes for normal and region mode.
+To customize YaTeX, user should use this function."
   (YaTeX-define-begend-key-normal key env map)
   (if YaTeX-inhibit-prefix-letter nil
     (YaTeX-define-begend-region-key
@@ -463,14 +464,14 @@ corresponding real arguments ARGS."
 
 ;;;###autoload
 (defun rindex (string char)
+  "Return the last position of STRING where character CHAR found."
   (let ((pos (1- (length string)))(index -1))
-    (while (>= pos 0)
-      (cond
-       ((= (aref string pos) char)
-	(setq index pos) (setq pos -1))
-       (t (setq pos (1- pos))))
-      )
-    index))
+    (catch 'rindex
+      (while (>= pos 0)
+	(cond
+	 ((= (aref string pos) char)
+	  (throw 'rindex pos))
+	 (t (setq pos (1- pos))))))))
 
 ;;;###autoload
 (defun point-beginning-of-line ()
@@ -487,7 +488,7 @@ corresponding real arguments ARGS."
 that gives the maximum value by the FUNC.  FUNC should take an argument
 of its window object.  Non-nil for optional third argument SELECT selects
 that window.  This function never selects minibuffer window."
-  (or (and (if (and YaTeX-emacs-19 select)
+  (or (and (if (and YaTeX-emacs-19 select window-system)
 	       (get-buffer-window buffer t)
 	     (get-buffer-window buffer))
 	   (progn
@@ -516,7 +517,7 @@ that window.  This function never selects minibuffer window."
 	  (or select (select-window window)))
 	 (t				;if one-window
 	  (cond
-	   ((and YaTeX-emacs-19 (get-buffer-window buffer t))
+	   ((and YaTeX-emacs-19 window-system (get-buffer-window buffer t))
 	    nil)			;if found in other frame
 	   (YaTeX-default-pop-window-height
 	    (split-window-calculate-height YaTeX-default-pop-window-height)
@@ -530,11 +531,15 @@ that window.  This function never selects minibuffer window."
 (cond
  ((fboundp 'screen-height)
   (fset 'YaTeX-screen-height 'screen-height)
-  (fset 'YaTeX-screen-width 'screen-width))
+  (fset 'YaTeX-screen-width 'screen-width)
+  (fset 'YaTeX-set-screen-height 'set-screen-height)
+  (fset 'YaTeX-set-screen-width 'set-screen-width))
  ((fboundp 'frame-height)
   (fset 'YaTeX-screen-height 'frame-height)
-  (fset 'YaTeX-screen-width 'frame-width))
- (t (error "I don't know how to run windows.el on this Emacs...")))
+  (fset 'YaTeX-screen-width 'frame-width)
+  (fset 'YaTeX-set-screen-height 'set-frame-height)
+  (fset 'YaTeX-set-screen-width 'set-frame-width))
+ (t (error "I don't know how to run YaTeX on this Emacs...")))
 
 ;;;###autoload
 (defun split-window-calculate-height (height)
@@ -664,6 +669,13 @@ See documentation of YaTeX-minibuffer-complete."
     (self-insert-command 1)
     (YaTeX-minibuffer-complete)))
 
+(defun YaTeX-yatex-buffer-list ()
+  (save-excursion
+    (delq nil (mapcar (function (lambda (buf)
+				  (set-buffer buf)
+				  (if (eq major-mode 'yatex-mode) buf)))
+		      (buffer-list)))))
+
 (defun foreach-buffers (pattern job)
   "For each buffer which matches with PATTERN, do JOB."
   (let ((list (buffer-list)))
@@ -740,6 +752,19 @@ If no such window exist, switch to buffer BUFFER."
     (read-with-history-in hsym prompt init))
    (t (read-string prompt init))))
 
+(defvar YaTeX-skip-next-reader-char ?\C-j)
+(defun YaTeX-read-string-or-skip (&rest args)
+  "Read string, or skip if last input char is \C-j."
+  (if (equal last-input-char YaTeX-skip-next-reader-char)
+      ""
+    (apply 'read-string args)))
+
+(defun YaTeX-completing-read-or-skip (&rest args)
+  "Do completing-read, or skip if last input char is \C-j."
+  (if (equal last-input-char YaTeX-skip-next-reader-char)
+      ""
+    (apply 'completing-read args)))
+
 ;;;###autoload
 (fset 'YaTeX-rassoc
       (if (and nil (fboundp 'rassoc) (subrp (symbol-function 'rassoc)))
@@ -820,6 +845,21 @@ NULL includes null string in a list."
   (or (featurep 'windows) (error "Why don't you use `windows.el'?"))
   (win-switch-to-window 1 (- last-command-char win:base-key)))
 
+;;;###autoload
+(defun YaTeX-command-to-string (cmd)
+  (if (fboundp 'shell-command-to-string)
+      (funcall 'shell-command-to-string cmd)
+    (let ((tbuf " *tmpout*"))
+      (if (get-buffer-create tbuf) (kill-buffer tbuf))
+      (let ((standard-output (get-buffer-create tbuf)))
+	(unwind-protect
+	    (save-excursion
+	      (call-process
+	       shell-file-name nil tbuf nil YaTeX-shell-command-option cmd)
+	      (set-buffer tbuf)
+	      (buffer-string))
+	  (kill-buffer tbuf))))))
+      
 ;;;###autoload
 (defun YaTeX-reindent (col)
   "Remove current indentation and reindento to COL column."
@@ -886,7 +926,7 @@ of 'YaTeX-inner-environment, which can be referred by
 		    (goto-char m0)
 		    (put 'YaTeX-inner-environment 'indent (current-column))
 		    (throw 'begin t)))))
-	  (buffer-substring
+	  (YaTeX-buffer-substring
 	   (progn (skip-chars-forward open) (1+ (point)))
 	   (progn (skip-chars-forward close) (point)))))))
 
@@ -972,9 +1012,16 @@ Optional third argument NOERR causes no error for unballanced environment."
   (let ((env (YaTeX-inner-environment)))
     (if (not env) (error "No premature environment")
       (save-excursion
-	(if (YaTeX-search-active-forward
-	     (YaTeX-replace-format-args YaTeX-struct-end env "" "")
-	     YaTeX-comment-prefix nil t)
+	(if (and
+	     (YaTeX-re-search-active-forward
+	      (concat
+	       "\\(" (YaTeX-replace-format-args
+		      YaTeX-struct-end env "" "")
+	       "\\)\\|\\(" (YaTeX-replace-format-args
+		      YaTeX-struct-begin env "" "")
+	       "\\)")
+	      YaTeX-comment-prefix nil t)
+	     (match-beginning 1))	;is closing struc.
 	    (if (y-or-n-p
 		 (concat "Environment `" env
 			 "' may be already closed. Force close?"))
@@ -1014,28 +1061,31 @@ to most recent sectioning command."
 (defun YaTeX-mark-environment ()
   "Mark current position and move point to end of environment."
   (interactive)
+  (require 'yatexmth)
   (let ((curp (point)))
-    (if (and (YaTeX-on-begin-end-p) (match-beginning 1)) ;if on \\begin
-	(progn (goto-char (match-end 0)))
-      (if (= (char-after (point)) ?\\) nil	;if on \\end
-	(skip-chars-backward "^\n\\\\")
-	(or (bolp) (forward-char -1))))
-    (if (not (YaTeX-end-of-environment))   ;arg1 turns to match-beginning 1
-	(progn
-	  (goto-char curp)
-	  (error "Cannot found the end of current environment."))
-      (YaTeX-goto-corresponding-environment)
-      (beginning-of-line)		;for confirmation
-      (if (< curp (point))
+    (if (YaTeX-in-math-mode-p)
+	(YaTeX-mark-mathenv)
+      (if (and (YaTeX-on-begin-end-p) (match-beginning 1)) ;if on \\begin
+	  (progn (goto-char (match-end 0)))
+	(if (= (char-after (point)) ?\\) nil ;if on \\end
+	  (skip-chars-backward "^\n\\\\")
+	  (or (bolp) (forward-char -1))))
+      (if (not (YaTeX-end-of-environment)) ;arg1 turns to match-beginning 1
 	  (progn
-	    (message "Mark this environment?(y or n): ")
-	    (if (= (read-char) ?y) nil
-	      (goto-char curp)
-	      (error "Abort.  Please call again at more proper position."))))
-      (set-mark-command nil)
-      (YaTeX-goto-corresponding-environment)
-      (end-of-line)
-      (if (eobp) nil (forward-char 1)))))
+	    (goto-char curp)
+	    (error "Cannot found the end of current environment."))
+	(YaTeX-goto-corresponding-environment)
+	(beginning-of-line)		;for confirmation
+	(if (< curp (point))
+	    (progn
+	      (message "Mark this environment?(y or n): ")
+	      (if (= (read-char) ?y) nil
+		(goto-char curp)
+		(error "Abort.  Please call again at more proper position."))))
+	(set-mark-command nil)
+	(YaTeX-goto-corresponding-environment)
+	(end-of-line)
+	(if (eobp) nil (forward-char 1))))))
 
 (defun YaTeX-kill-buffer (buffer)
   "Make effort to show parent buffer after kill."
@@ -1048,6 +1098,57 @@ to most recent sectioning command."
 	 (get-file-buffer pf)
 	 (switch-to-buffer (get-file-buffer pf)))))
 
+(defun YaTeX-getset-builtin (key &optional value)
+  "Read source built-in command of %# usage."
+  (catch 'builtin
+    (let*((bl (delq nil (list (current-buffer)
+			      (and YaTeX-parent-file
+				   (get-file-buffer YaTeX-parent-file)))))
+	  (tuple (cdr (assq major-mode
+			    '((yatex-mode "%#" . "\n")
+			      (yahtml-mode "<!-- #" . "[ \t]*-->\\|\n")))))
+	  (leader (or (car tuple) ""))
+	  (closer (or (cdr tuple) ""))
+	  (prompt (format "Built-in for %s: " key)))
+      (save-excursion
+	(while bl
+	  (set-buffer (car bl))
+	  (save-excursion
+	    (goto-char (point-min))
+	    (if (and (re-search-forward
+		      (concat "^" (regexp-quote (concat leader key))) nil t)
+		     (not (eolp)))
+		(throw 'builtin
+		       (let (b e w)
+			 (skip-chars-forward " \t" (point-end-of-line))
+			 (setq b (point)
+			       e (if (re-search-forward closer nil t)
+				     (match-beginning 0)
+				   (point-end-of-line))
+			       w (YaTeX-buffer-substring b e))
+			 (if (null value)
+			     w
+			   (delete-region b e)
+			   (goto-char b)
+			   (if (symbolp value)
+			       (setq value (read-string prompt w)))
+			   (insert value)
+			   value)))))
+	  (setq bl (cdr bl)))
+	; not found
+	(if (null value)
+	    nil				;not set mode, return simply nil
+	  (if (symbolp value)
+	      (setq value (read-string prompt)))
+	  (save-excursion
+	    (goto-char (point-min))
+	    (insert leader key " " value "\n")
+	    value))))))			;on set mode, return set value
+
+(defun YaTeX-get-builtin (key)
+  "Read source built-in command of %# usage."
+  (YaTeX-getset-builtin key))
+
 ;;;VER2
 (defun YaTeX-insert-struc (what env)
   (cond
@@ -1069,6 +1170,24 @@ to most recent sectioning command."
    ((fboundp 'truncate-string) (truncate-string str width))
    (t (substring str 0 width))))
 
+(defun YaTeX-hex (str)
+  "Return int expressed by hexadecimal string STR."
+  (if (string< "20" emacs-version)
+      (string-to-number str 16)
+    (let ((md (match-data)))
+      (unwind-protect
+	  (if (string-match "[^0-9a-f]" str)
+	      (error "Non hexadecimal character in %s" str)
+	    (let ((i 0) d)
+	      (setq str (downcase str))
+	      (while (string< "" str)
+		(setq d (+ 0 (string-to-char str)) ; + 0 for XEmacs
+		      i (+ (* 16 i) (- d (if (<= d ?9) ?0 (- ?a 10))))
+		      str (substring str 1)))
+	      i))
+	(store-match-data md)))))
+
+
 ;;; Function for menu support
 (defun YaTeX-define-menu (keymap bindlist)
   "Define KEYMAP(symbol)'s menu-bindings according to BINDLIST.
@@ -1404,6 +1523,26 @@ compared by regexp."
 	    (throw 'found (car alist)))
 	(setq alist (cdr alist))))))
 
+(defun YaTeX-push-to-kill-ring (string)
+  "Push STRING to kill-ring, then show guidance message."
+  (and (stringp string) (string< "" string)
+       (let ((key (key-description (where-is-internal 'yank nil t)))
+	     (msg
+	      (if YaTeX-japan
+		  " kill-ringɓ܂Byank(%s)œ\tł܂"
+		" is stored into kill-ring.  Paste it by yank(%s).")))
+	 (kill-new string)
+	 (message (concat "`%s'" msg) string key))))
+
+(defun YaTeX-elapsed-time (before after)
+  "Get elapsed time from BEFORE and AFTER, which are given from currente-time."
+  (if (fboundp 'float)			;Then, current-time function should be.
+      (let ((mil (float 1000000)))	;To protect parse error before 19
+	(+ (* (- (nth 0 after) (nth 0 before)) 65536)
+	   (- (nth 1 after) (nth 1 before))
+	   (- (/ (nth 2 after) mil)
+	      (/ (nth 2 before) mil))))))
+
 ;;;
 ;; Functions for the Installation time
 ;;;
@@ -1429,7 +1568,15 @@ compared by regexp."
 		 (lambda (arg)
 		   (find-file arg)
 		   (texinfo-format-buffer)
-		   (basic-save-buffer)))
+		   (cond
+		    ((fboundp 'set-buffer-file-coding-system)
+		     (set-buffer-file-coding-system 'sjis-dos))
+		    ((fboundp 'set-file-coding-system)
+		     (set-file-coding-system '*sjis*dos))
+		    ((boundp 'NEMACS)
+		     (set (make-local-variable 'kanji-fileio-code) 1)))
+		   (let ((coding-system-for-write buffer-file-coding-system))
+		     (basic-save-buffer))))
 		command-line-args-left)
 	(kill-emacs))))
 
