@@ -1,9 +1,9 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX math-mode-specific functions.
 ;;; yatexmth.el
-;;; (c)1993-2003 by HIROSE Yuuji [yuuji@yatex.org]
-;;; Last modified Sun Mar 30 19:56:00 2003 on firestorm
-;;; $Id: yatexmth.el,v 1.71 2003/05/01 13:38:49 yuuji Rel $
+;;; (c)1993-2012 by HIROSE Yuuji [yuuji@yatex.org]
+;;; Last modified Tue Jan 24 08:58:56 2012 on firestorm
+;;; $Id: yatexmth.el,v 1.76 2012/05/14 10:55:57 yuuji Rel $
 
 ;;; [Customization guide]
 ;;;
@@ -107,6 +107,11 @@
 (YaTeX-setq-math-sym YaTeX-image-neg		""		"")
 (YaTeX-setq-math-sym YaTeX-image-flat		""		"")
 (YaTeX-setq-math-sym YaTeX-image-sqrt		""		"")
+(defvar YaTeX-image-nearrow '("__\n /|\n/" "  Q\n  ^|\n^" ))
+(defvar YaTeX-image-nwarrow '(" __\n|\\\n  \\" " Q\n|_\n   _"))
+(defvar YaTeX-image-searrow '("\\\n \\|\n--`" "_\n  _|\n  P"))
+(defvar YaTeX-image-swarrow '("  /\n|/\n'~~" "   ^\n|^\n P"))
+
 
 (defvar
  YaTeX-math-sign-alist-default
@@ -147,7 +152,7 @@
    ("\\\\"	"setminus"	"\\")
    (")("	"wr"		" )\n(")
    ("<>"	"diamond"	"<>")
-   ("/\-"	"bigtriangleup"	("/\\\n~~" ""))
+   ("/\\-"	"bigtriangleup"	("/\\\n~~" ""))
    ("-\\/"	"bigtriangledown" ("__\n\\/" ""))
    ("<|"	"triangleleft"	"<|")
    ("|>"	"triangleright"	"|>")
@@ -166,8 +171,11 @@
    ("II"	"amalg"		"II")
    ;	:
    ;;relational operators
-   ("<"		"leq"		("<\n-"		""))
-   (">"		"geq"		(">\n-"		""))
+   ("<"		"leq"		("<\n-"		"<\n-"))
+   ("=<"	"leqq"		("<\n="		""))
+   (">"		"geq"		(">\n-"		">\n-"))
+   (">="	"geqq"		(">\n="		""))
+   ("=:"	"fallingdotseq"	(".\n==\n ."	""))
    ("-="	"equiv"		("=\n-"		YaTeX-image-equiv))
    ("=-"	"equiv"		("=\n-"		YaTeX-image-equiv))
    ("---"	"equiv"		("=\n-"		YaTeX-image-equiv))
@@ -202,6 +210,9 @@
    ("-<"	"prec"		("-<"		""))
    ("-<="	"preceq"	("-<\n-"	"\n="))
    ("<<"	"ll"		("<<"		"s"))
+   ("<<"	"lll"		"<<<")
+   (">>"	"gg"		(">>"		"t"))
+   (">>>"	"ggg"		">>>")
    ;	:
    ;;arrows
    ("<-"	"leftarrow"	("<-"		""))
@@ -224,6 +235,18 @@
    ("\C-n"	"downarrow"	("|\nv" ""))
    ("v|"	"downarrow"	("|\nv" ""))
    ("v||"	"Downarrow"	"||\n\\/")
+   ("\C-p\C-f"	"nearrow"	YaTeX-image-nearrow)
+   ("\C-f\C-p"	"nearrow"	YaTeX-image-nearrow)
+   ("ne"	"nearrow"	YaTeX-image-nearrow)
+   ("\C-p\C-b"	"nwarrow"	YaTeX-image-nwarrow)
+   ("\C-b\C-p"	"nwarrow"	YaTeX-image-nwarrow)
+   ("nw"	"nwarrow"	YaTeX-image-nwarrow)
+   ("\C-n\C-f"	"searrow"	YaTeX-image-searrow)
+   ("\C-f\C-n"	"searrow"	YaTeX-image-searrow)
+   ("se"	"searrow"	YaTeX-image-searrow)
+   ("\C-n\C-b"	"swarrow"       YaTeX-image-swarrow)
+   ("\C-b\C-n"	"swarrow"	YaTeX-image-swarrow)
+   ("sw"	"swarrow"	YaTeX-image-swarrow)
    ("|->"	"mapsto"	("|->"		"|"))
    ("<-)"	"hookleftarrow"	("   ,\n<--+"	"   R\n<--/"))
    ("(->"	"hookrightarrow" ("`\n+-->"	"^\n_-->"))
@@ -259,6 +282,8 @@
    ("O+"	"bigoplus"	"/~~~\\\n| + |\n\\___/")
    ("Ox"	"bigotimes"	"/~~~\\\n| X |\n\\___/")
    ;;other marks
+   ("angle"	"angle"		("/\n~" ""))
+   ("/_"	"angle"		("/\n~" ""))
    ("Z"		"aleph"		"|\\|")
    ("|\\|"	"aleph"		"|\\|")
    ("h-"	"hbar"		"_\nh")
@@ -282,12 +307,21 @@
    ("partial"	"partial"	(" -+\n+-+\n+-+" YaTeX-image-partial))
    ("round"	"partial"	(" -+\n+-+\n+-+" YaTeX-image-partial))
    ("[]"	"Box"		"[]")
+   ("no"	"notag"		"\\notag")
+   (":"		"colon"		":")
    ("Diamond"	"Diamond"	"/\\\n\\/")
    ("3"		"triangle"	"/\\\n~~")
    ("C"		"clubsuit"	" o\no+o\n |")
    ("D"		"diamondsuit"	"/\\\n\\/")
    ("H"		"heartsuit"	"<^^>\n \\/")
    ("S"		"spadesuit"	" /\\\n<++>\n /\\")
+   ("mi"	"mathit"	"\\mathit{}")
+   ("mr"	"mathrm"	"\\mathrm{}")
+   ("mb"	"mathbf"	"\\mathbf{}")
+   ("mt"	"mathtt"	"\\mathtt{}")
+   ("ms"	"mathsf"	"\\mathsf{}")
+   ("mc"	"mathcal"	"\\mathcal{}")
+   ("mn"	"mathnormal"	"\\mathnormal{}")
    )
  "Default LaTeX-math-command alist.")
 
@@ -380,8 +414,8 @@
 (defvar YaTeX-math-display-width
   (* 8 (1+ (/ YaTeX-math-sign-width 8))))
 (defvar YaTeX-math-menu-map nil
-  "Keymap used in YaTeX mathematical sign menu mode."
-)
+  "Keymap used in YaTeX mathematical sign menu mode.")
+
 (if YaTeX-math-menu-map nil
   (setq YaTeX-math-menu-map (make-sparse-keymap))
   (define-key YaTeX-math-menu-map "n"	'next-line)
@@ -400,17 +434,18 @@
   (define-key YaTeX-math-menu-map "q"	'abort-recursive-edit))
 
 (defvar YaTeX-math-exit-key "\e"
-  "*Key sequence after prefix key of YaTeX-math-mode to exit from math-mode."
-)
+  "*Key sequence after prefix key of YaTeX-math-mode to exit from math-mode.")
 
 (defmacro YaTeX-math-japanese-sign (list)
   (list 'nth 1 list))
 
 (defvar YaTeX-math-cmd-regexp (concat (regexp-quote YaTeX-ec) "[A-z|]"))
-(defvar YaTeX-math-verbatim-environments
-  '("alltt")
-  "*List of environments in which LaTeX math mode is disabled.
-This value is appended with YaTeX-verbatim-environments.")
+
+;;;	alltt goes into YaTeX-verbatim-environments 2011/3/16
+;;(defvar YaTeX-math-verbatim-environments
+;;  '("alltt")
+;;  "*List of environments in which LaTeX math mode is disabled.
+;;This value is appended with YaTeX-verbatim-environments.")
 
 ;;;
 ;;YaTeX math-mode functions
@@ -438,8 +473,7 @@ This value is appended with YaTeX-verbatim-environments.")
 		   " $'")))
       (setq YaTeX-math-mode nil)
       (message "Exit from math mode."))
-    (set-buffer-modified-p (buffer-modified-p)))
-)
+    (set-buffer-modified-p (buffer-modified-p))))
 
 (defun YaTeX-math-forward (arg)
   (interactive "p")
@@ -460,77 +494,109 @@ This value is appended with YaTeX-verbatim-environments.")
     (YaTeX-math-gets sign))
    ((symbolp sign)
     (YaTeX-math-gets (symbol-value sign)))
-   (t sign))
-)
+   (t sign)))
 
 (defun YaTeX-math-get-sign (list)
-  (YaTeX-math-gets (car (cdr-safe (cdr-safe list))))
-)
+  (YaTeX-math-gets (car (cdr-safe (cdr-safe list)))))
+
 (defvar YaTeX-math-section-type-regexp
   "eqn\\\\\\sw+\\b"
   "*Regexp of section-type math-mode macro")
 
 (defun YaTeX-in-math-mode-p ()
-  "If current position is supposed to be in LaTeX-math-mode, return t."
-  (or (YaTeX-quick-in-environment-p
-       (append
-	'("math" "eqnarray" "equation" "eqnarray*" "displaymath");LaTeX
-	(if YaTeX-use-AMS-LaTeX
-	    ;; And math modes of AMS-LaTeX
-	    '("align" "align*" "split" "multline" "multline*" "gather"
-	      "gather*" "aligned*" "gathered" "gathered*" "alignat"
-	      "equation*" "cases" 
-	      "alignat*" "xalignat" "xalignat*" "xxalignat" "xxalignat*"))))
-      (let*((p (point)) (nest 0) me0 r
-	    (delim (concat YaTeX-sectioning-regexp "\\|^$\\|^\C-l"))
-	    (boundary
-	     (save-excursion
-	       (if (looking-at delim)
-		   (goto-char (max (point-min) (1- (point)))))
-	       (re-search-backward delim nil 1)
-	       (point))))
-	(save-excursion
-	  (cond
-	   ((catch 'open
-	      (save-excursion
-		(while (and (>= nest 0)
-			    (re-search-backward
-			     (concat YaTeX-ec-regexp	;\
-				     "\\([()]\\|[][]\\)") boundary t))
-		  (setq me0 (match-end 0))
-		  (if (or (YaTeX-on-comment-p)
-			  (YaTeX-literal-p)) nil
-		    (if (or (= (char-after (1- me0)) ?\))
-			    (= (char-after (1- me0)) ?\]))
-			(setq nest (1+ nest))
-		      (if (= (preceding-char) ?\\ ) nil;;\\[5pt]
-			(setq nest (1- nest))))))
-		(if (< nest 0) (throw 'open t))))
-	    t)
-	   ((and (setq r (YaTeX-on-section-command-p
-			  YaTeX-math-section-type-regexp))
-		 (numberp r)
-		 (> r 0))
-	    t)
-	   (t (catch 'dollar
-		(while ;(search-backward "$" boundary t);little bit fast.
-		    (YaTeX-re-search-active-backward ;;;;;; Too slow???
-		     "\\$" (concat "[^\\\\]" YaTeX-comment-prefix) boundary t)
-		  (cond
-		   ((equal (char-after (1- (point))) ?$) ; $$ equation $$
-		    (backward-char 1)
-		    (setq nest (1+ nest)))
-		   ((let ((YaTeX-verbatim-environments
-			   (append YaTeX-math-verbatim-environments
-				   YaTeX-verbatim-environments)))
-		      (YaTeX-literal-p))
-		    nil)
-		   ((and (equal (char-after (1- (point))) ?\\ )
-			 (not (equal (char-after (- (point) 3)) ?\\ )))
-		    nil)		;\$
-		   (t (setq nest (1+ nest)))))
-		(if (= (% nest 2) 1) (throw 'dollar t))))))))
-)
+  "If current position is supposed to be in LaTeX-math-mode, return t.
+This function refers a local variable `source-window' in YaTeX-make-section."
+  (save-excursion
+    (and (boundp 'source-window) source-window
+	 (set-buffer (window-buffer source-window)))
+    (or (YaTeX-quick-in-environment-p
+	 (append
+	  '("math" "eqnarray" "equation" "eqnarray*" "displaymath") ;LaTeX
+	  (if YaTeX-use-AMS-LaTeX
+	      ;; And math modes of AMS-LaTeX
+	      ;;'("align" "align*" "split" "multline" "multline*" "gather"
+	      ;;  "gather*" "aligned*" "gathered" "gathered*" "alignat"
+	      ;;  "equation*" "cases" "flalign" "flalign*"
+	      ;;  "alignat*" "xalignat" "xalignat*" "xxalignat" "xxalignat*"
+	      YaTeX-math-begin-list
+	    )))
+	(let*((p (point)) (nest 0) me0 r firstp dollar
+	      (delim (concat YaTeX-sectioning-regexp "\\|^$\\|^\C-l"))
+	      (boundary
+	       (save-excursion
+		 (if (looking-at delim)
+		     (goto-char (max (point-min) (1- (point)))))
+		 (re-search-backward delim nil 1)
+		 (point))))
+	  (save-excursion
+	    (cond
+	     ((catch 'open
+		(save-excursion
+		  (while (and (>= nest 0)
+			      (re-search-backward
+			       (concat YaTeX-ec-regexp ;\
+				       "\\([()]\\|[][]\\)") boundary t))
+		    (setq me0 (match-end 0))
+		    (if (or (YaTeX-on-comment-p)
+			    (YaTeX-literal-p)) nil
+		      (if (or (= (char-after (1- me0)) ?\))
+			      (= (char-after (1- me0)) ?\]))
+			  (setq nest (1+ nest))
+			(if (= (preceding-char) ?\\ ) nil ;;\\[5pt]
+			  (setq nest (1- nest))))))
+		  (if (< nest 0)
+		      (throw 'open (cons (YaTeX-match-string 0) (point)))))))
+	     ((and (setq r (YaTeX-on-section-command-p
+			    YaTeX-math-section-type-regexp))
+		   (numberp r)
+		   (> r 0))
+	      t)
+	     (t (catch 'dollar
+		  (while ;(search-backward "$" boundary t);little bit fast.
+		      (YaTeX-re-search-active-backward ;;;;;; Too slow???
+		       "\\$" (concat "[^\\\\]" YaTeX-comment-prefix) boundary t)
+		    (setq dollar "$")
+		    (cond
+		     ((equal (char-after (1- (point))) ?$) ; $$ equation $$
+		      (setq dollar "$$")
+		      (backward-char 1)
+		      (setq nest (1+ nest)))
+		     ((YaTeX-literal-p)
+		      nil)
+		     ((and (equal (char-after (1- (point))) ?\\ )
+			   (not (equal (char-after (- (point) 3)) ?\\ )))
+		      nil)		;\$
+		     (t (setq nest (1+ nest))))
+		    (if (and (= nest 1) (null firstp))
+			(setq firstp (cons dollar (point)))))
+		  (if (= (% nest 2) 1)
+		      (throw 'dollar firstp))))))))))
+
+(defun YaTeX-mark-mathenv ()
+  "Mark current mathematic environment."
+  (interactive)
+  (let ((mmp (YaTeX-in-math-mode-p)) type bpt)
+    (if (or (null mmp) (not (stringp (setq type (car mmp)))))
+	nil				;if nil or not string, do nothing
+     (setq bpt (cdr mmp))
+     (goto-char bpt)
+     (cond
+      ((string-match "\\$" type)
+       (set-mark-command nil)
+       (skip-chars-forward "$")
+       (YaTeX-search-active-forward
+	type YaTeX-comment-prefix nil)	;if it cause error, obey it
+       (goto-char (match-end 0)))
+      ;;
+      ((string-match "^\\\\[\\[(]" type)
+       (set-mark-command nil)
+       (YaTeX-goto-corresponding-leftright)
+       (skip-chars-forward "])\\\\"))
+      ;;
+      ((string-match "^[a-z]" type)	; \begin\end type math
+       (set-mark-command nil)
+       (YaTeX-goto-corresponding-environment)
+       (goto-char (match-end 0)))))))
 
 (defun YaTeX-math-display-list (list cols)
   (goto-char (point-max))
@@ -576,7 +642,7 @@ This value is appended with YaTeX-verbatim-environments.")
 	    (progn (YaTeX-math-display-list (car list) cols)
 		   (setq cols (% (1+ cols) maxcols))))
 	(setq list (cdr list)))
-      (goto-line 4)
+      (goto-char (point-min)) (forward-line 3)
       (use-local-map YaTeX-math-menu-map)
       (setq buffer-read-only t)
       (unwind-protect
@@ -589,8 +655,7 @@ This value is appended with YaTeX-verbatim-environments.")
 		   (prog2 (skip-chars-forward "^ \t\n") (point)))
 		nil))
 	(kill-buffer YaTeX-math-menu-buffer))
-      command))
-)
+      command)))
 
 ;
 (defun YaTeX-math-show-image (image &optional exit-char)
@@ -702,8 +767,8 @@ at least you get to read the beginning."
 
 	      (if match
 		  (progn (delete-region beg (point))
-			 (setq single-command (car (cdr match)))
-			 (insert YaTeX-ec single-command)
+			 (setq YaTeX-single-command (car (cdr match)))
+			 (insert YaTeX-ec YaTeX-single-command)
 			 (if (and YaTeX-math-need-image
 				  (setq sign (YaTeX-math-get-sign match)))
 			     (YaTeX-math-show-image (concat sign "\n")))
@@ -716,14 +781,14 @@ at least you get to read the beginning."
       (if (eq result t)
 	  (setq unread-command-char last-char)
 	(message "Done."))
-      (if (assoc single-command section-table)
-	  (YaTeX-make-section nil nil nil single-command)
+      (if (assoc YaTeX-single-command section-table)
+	  (YaTeX-make-section nil nil nil YaTeX-single-command)
 	(setq YaTeX-current-completion-type 'maketitle)
-	(YaTeX-make-singlecmd single-command)))
+	(YaTeX-make-singlecmd YaTeX-single-command)))
      ((eq result 'abort)
       (message "Abort."))
      ((eq result 'escape)
-      (call-interactively (global-key-binding this-key)))
+      (call-interactively (lookup-key global-map this-key)))
      ((eq result 'exit)
       (YaTeX-toggle-math-mode))
      ((eq result 'menu)
@@ -744,7 +809,7 @@ If so return the cons of its invocation key and image-string."
 	  (setq list (cdr list)))
 	(setq lists (cdr lists))))))
 
-;;; ----- for AMS LaTeX (by matsu@math.s.chiba-u.ac.jp) -----
+;;; ----- for AMS LaTeX (by matsu<at>math.s.chiba-u.ac.jp) -----
 (defvar YaTeX-ams-paren-modifier
   '(("Biggl" . "Biggr") ("biggl" . "biggr")
     ("Bigl" . "Bigr") ("bigl" . "bigr")
@@ -782,7 +847,7 @@ If so return the cons of its invocation key and image-string."
 				       (+ (point) 3 longest) t))
 	  (progn
 	    (setq move (- (point) point))
-	    (setq paren (match-string 0))
+	    (setq paren (YaTeX-match-string 0))
 	    (setq list YaTeX-ams-paren-modifier)
 	    ;; criterion for whether on [] () \{\} or not.
 	    (if (string-match YaTeX-left-paren paren)
@@ -830,8 +895,8 @@ If so return the cons of its invocation key and image-string."
 						 paren-regexp "\\)")))
 			(setq flag nil)))
 		  (setq list (cdr list)))))
-	    (if (<= move (length (match-string 0)))
-		(match-string 0)))))))
+	    (if (<= move (length (YaTeX-match-string 0)))
+		(YaTeX-match-string 0)))))))
 
 (defun YaTeX-goto-open-paren (&optional jumpto-co)
   "Jump to the exact position of open parenthesis.
@@ -872,8 +937,8 @@ If optional argument JUMPTO-CO is non-nil, goto corresponding parentheses."
   (interactive)
   (if (not (and YaTeX-use-AMS-LaTeX (YaTeX-on-parenthesis-p)))
       nil
-    (let* ((mod (match-string 1)) ;; modifier
-	   (paren (if mod (match-string 2) (match-string 0))) ;; paren
+    (let* ((mod (YaTeX-match-string 1)) ;; modifier
+	   (paren (if mod (YaTeX-match-string 2) (YaTeX-match-string 0))) ;; paren
 	   (mod-length (if (or (string= mod "\\left") (string= mod "\\right"))
 			   5            ;; 5 in case left or right
 			 (length mod))) ;; length of modifier
