@@ -1,9 +1,9 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX sectioning browser.
 ;;; yatexsec.el
-;;; (c ) 1994,1998,1999,2003 by HIROSE Yuuji [yuuji@yatex.org]
-;;; Last modified Wed Mar  5 13:19:11 2003 on firestorm
-;;; $Id: yatexsec.el,v 1.71 2003/05/01 13:38:49 yuuji Rel $
+;;; (c) 1994-2012 by HIROSE Yuuji [yuuji@yatex.org]
+;;; Last modified Mon Jan  9 20:20:32 2012 on firestorm
+;;; $Id: yatexsec.el,v 1.76 2012/05/14 10:55:57 yuuji Rel $
 
 (defvar YaTeX-sectioning-level
   '(("part" . 0)
@@ -133,7 +133,8 @@ This must be the heighest number in YaTeX-sectioning-level.")
 	  (or
 	   (and ln (string< "" ln)
 		(progn
-		  (goto-line (max 1 (1- (string-to-int ln))))
+		  (goto-char (point-min))
+		  (forward-line (max 0 (- (string-to-int ln) 2)))
 		  (and
 		   (search-forward ptn nil t)
 		   (goto-char (match-beginning 0)))))
